package com.schbrain.canal.client.transfer;

import com.alibaba.otter.canal.client.CanalConnector;
import com.alibaba.otter.canal.protocol.CanalEntry;
import com.schbrain.canal.client.annotation.ListenPoint;
import com.schbrain.canal.client.conf.CanalClientConfig;
import com.schbrain.canal.client.core.HandlerConf;
import org.apache.commons.lang3.StringUtils;

import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.function.Predicate;

/**
 * @author zhuyf
 * @date 2022/6/16
 */
public class DefaultMessageTransponder extends AbstractBasicMessageTransponder{

    public DefaultMessageTransponder(CanalConnector connector,
                                     Map.Entry<String, CanalClientConfig> config, HandlerConf handlerConf) {
        super(connector, config,handlerConf);
    }

    @Override
    protected Predicate<Map.Entry<Method, ListenPoint>> getAnnotationFilter(String destination,CanalEntry.Header header, CanalEntry.EventType eventType) {
        Predicate<Map.Entry<Method, ListenPoint>> df = (e->{
            if(StringUtils.isBlank(e.getValue().destination())){
                return true;
            }
            return e.getValue().destination().equals(destination);
        });
        Predicate<Map.Entry<Method, ListenPoint>> sf = (e->{
            if(e.getValue().schema() == null || e.getValue().schema().length ==0){
                return true;
            }
            String schemaName = header.getSchemaName();
            for (String s : e.getValue().schema()) {
                if(schemaName.equals(s)){
                    return true;
                }
            }
            return false;
        });
        Predicate<Map.Entry<Method, ListenPoint>> tf = (e->{
            if(e.getValue().table() == null || e.getValue().table().length ==0){
                return true;
            }
            String tableName = header.getTableName();
            for (String s : e.getValue().table()) {
                if(tableName.equals(s)){
                    return true;
                }
            }
            return false;
        });
        Predicate<Map.Entry<Method, ListenPoint>> ef = (e->{
            if(e.getValue().eventType() == null || e.getValue().eventType().length ==0){
                return true;
            }
            for (CanalEntry.EventType eventType1 : e.getValue().eventType()) {
                if(eventType1 == eventType){
                    return true;
                }
            }
            return false;
        });
        return df.and(sf).and(tf).and(ef);
    }

    @Override
    protected Object[] getInvokeArgs(Method method,CanalEntry.Header header,CanalEntry.EventType eventType, CanalEntry.RowData rowData) {
        Class<?>[] classes = method.getParameterTypes();
        Object[] args = new Object[classes.length];
        for (int i = 0; i < classes.length; i++) {
            Class<?> p = classes[i];
            Object arg = null;
            if(p == CanalEntry.EventType.class){
                arg = eventType;
            }else if(p == CanalEntry.Header.class){
                arg = header;
            }else if(p == CanalEntry.RowData.class){
                arg = rowData;
            }
            args[i]  = arg;
        }
        return args;
    }


    @Override
    protected List<CanalEntry.EntryType> getIgnoreEntryTypes() {
        return Arrays.asList(CanalEntry.EntryType.TRANSACTIONBEGIN, CanalEntry.EntryType.TRANSACTIONEND, CanalEntry.EntryType.HEARTBEAT);
    }
}
